/*
 *
 * Copyright (C) 2013 Kerio Technologies s.r.o
 *
 * This file is part of libdkim++.
 *
 * libdkim++ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libdkim++ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libdkim++.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include "Exception.hpp"
#include "DKIM.hpp"

#include <list>
#include <string>

#include <openssl/evp.h>
#include <openssl/pem.h>
#include <openssl/err.h>

namespace DKIM
{
	class DLL_DECLARATION ISignatoryOptions
	{
		public:
			virtual ~ISignatoryOptions() {}

			virtual ISignatoryOptions* SetPrivateKey(RSA* privatekey) = 0;
			virtual ISignatoryOptions* SetPrivateKey(const std::string& privatekey) = 0;
			virtual ISignatoryOptions* SetDomain(const std::string& domain) = 0;
			virtual ISignatoryOptions* SetSelector(const std::string& selector) = 0;
			virtual ISignatoryOptions* SetAlgorithm(Algorithm algorithm) = 0;
			virtual ISignatoryOptions* SetHeaders(const std::list<std::string>& headers) = 0;
			virtual ISignatoryOptions* AddHeaders(const std::list<std::string>& headers) = 0;
			virtual ISignatoryOptions* SetSignBodyLength(unsigned long bodylength) = 0;
			virtual ISignatoryOptions* SetCanonModeHeader(CanonMode mode) = 0;
			virtual ISignatoryOptions* SetCanonModeBody(CanonMode mode) = 0;

			virtual EVP_PKEY* GetPrivateKey() const = 0;
			virtual const std::string& GetDomain() const = 0;
			virtual const std::string& GetSelector() const = 0;
			virtual Algorithm GetAlgorithm() const = 0;
			virtual const std::list<std::string>& GetHeaders() const = 0;
			virtual bool GetSignEmptyHeaders() const = 0;
			virtual unsigned long GetBodyLength() const = 0;
			virtual bool GetBodySignLength() const = 0;
			virtual CanonMode GetCanonModeHeader() const = 0;
			virtual CanonMode GetCanonModeBody() const = 0;

			static ISignatoryOptions* create();
			static void destroy(ISignatoryOptions* signatory);
	};

};
