/*
 *
 * Copyright (C) 2013 Kerio Technologies s.r.o
 *
 * This file is part of libdkim++.
 *
 * libdkim++ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libdkim++ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libdkim++.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include "Exception.hpp"
#include "DKIM.hpp"
#include "MailParser.hpp"

#include <string>

#include <openssl/evp.h>
#include <openssl/pem.h>
#include <openssl/err.h>

#include "ISignatory.hpp"

namespace DKIM
{
	class Signatory1 : public ISignatory
	{
		public:
			Signatory1(const ISignatoryOptions* options, bool doubleDots = false);
			Signatory1(const Signatory1& s);
			virtual ~Signatory1();

			virtual void beginSignature(std::istream& file) declareThrow (DKIM::PermanentError);
			virtual void addHeaderToFront(const std::string& header);
			virtual std::string getSignatureHeader()  declareThrow (DKIM::PermanentError);
			virtual ISignatory* clone() const;
		private:
			std::string computeDkimSignature() declareThrow (DKIM::PermanentError);

			Message m_msg;

			bool m_doubleDots;
			const ISignatoryOptions* m_options;
			std::string m_bodySignature;
			std::string m_dkimSignature;
			bool m_addedHeader;
	};
};
