/*
 *
 * Copyright (C) 2009,2010,2011 Halon Security <support@halon.se>
 *
 * This file is part of libdkim++.
 *
 * libdkim++ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libdkim++ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libdkim++.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _DKIM_SIGNATORYOPTIONS_HPP_
#define _DKIM_SIGNATORYOPTIONS_HPP_

#include "Exception.hpp"
#include "DKIM.hpp"
#include "MailParser.hpp"

using DKIM::Message;

#include "Base64.hpp"

using DKIM::Conversion::Base64;

#include <list>
#include <string>

#include <openssl/evp.h>
#include <openssl/pem.h>
#include <openssl/err.h>

#include "ISignatoryOptions.hpp"

namespace DKIM
{
	class SignatoryOptions : public ISignatoryOptions
	{
		public:
			SignatoryOptions();
			virtual ~SignatoryOptions();

			virtual SignatoryOptions* SetPrivateKey(RSA* privatekey);
			virtual SignatoryOptions* SetPrivateKey(const std::string& privatekey);
			virtual SignatoryOptions* SetDomain(const std::string& domain);
			virtual SignatoryOptions* SetSelector(const std::string& selector);
			virtual SignatoryOptions* SetAlgorithm(Algorithm algorithm);
			virtual SignatoryOptions* SetHeaders(const std::list<std::string>& headers);
			virtual SignatoryOptions* AddHeaders(const std::list<std::string>& headers);
			virtual SignatoryOptions* SetSignBodyLength(unsigned long bodylength);
			virtual SignatoryOptions* SetCanonModeHeader(CanonMode mode);
			virtual SignatoryOptions* SetCanonModeBody(CanonMode mode);

			virtual EVP_PKEY* GetPrivateKey() const
			{ return m_privateKey; }
			virtual const std::string& GetDomain() const
			{ return m_domain; }
			virtual const std::string& GetSelector() const
			{ return m_selector; }
			virtual Algorithm GetAlgorithm() const
			{ return m_algorithm; }
			virtual const std::list<std::string>& GetHeaders() const
			{ return m_headers; }
			virtual bool GetSignEmptyHeaders() const
			{ return m_signEmptyHeaders; }
			virtual unsigned long GetBodyLength() const
			{ return m_bodyLength; }
			virtual bool GetBodySignLength() const
			{ return m_bodySignLength; }
			virtual CanonMode GetCanonModeHeader() const
			{ return m_canonHead; }
			virtual CanonMode GetCanonModeBody() const
			{ return m_canonBody; }
		private:
			void freeKey();

			EVP_PKEY* m_privateKey;
			RSA* m_rsa;

			std::string m_domain;
			std::string m_selector;

			Algorithm m_algorithm;

			std::list<std::string> m_headers;
			bool m_signEmptyHeaders;

			unsigned long m_bodyLength;
			bool m_bodySignLength;

			CanonMode m_canonHead;
			CanonMode m_canonBody;
	};
};

#endif
