/*
 *
 * Copyright (C) 2013 Kerio Technologies s.r.o
 *
 * This file is part of libdkim++.
 *
 * libdkim++ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libdkim++ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libdkim++.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "SignatureChecker.hpp"

#include "Exception.hpp"
#include "Validatory.hpp"

namespace DKIM {

DkimSignatureStatus checkSignature(std::string& signedDomain, std::istream& mail, CustomDNSResolver resolver)
{
	Validatory validatory(mail);
	if (resolver) {
		validatory.CustomDNSResolver = resolver;
	}
	DkimSignatureStatus dkimResult = DkimSignatureNone;

	Validatory::SignatureList::const_iterator i = validatory.GetSignatures().begin();
	if (i != validatory.GetSignatures().end()) {
		DKIM::PublicKey pub;
		DKIM::Signature sig;
		try {
			validatory.GetSignature(i, sig);
			signedDomain = sig.GetDomain();
			validatory.GetPublicKey(sig, pub);
			validatory.CheckSignature(i, sig, pub);
			dkimResult = DkimSignatureValid;
		} catch (DKIM::TemporaryError&) {
			dkimResult = DkimSignatureDnsFail;
		} catch (DKIM::PermanentError&) {
			if (pub.SoftFail())
				dkimResult = DkimSignatureValid;
			else
				dkimResult = DkimSignatureInvalid;
		}
	}

	return dkimResult;
}

}
